package com.yuyu.srwildentity.config.condition;

import net.md_5.bungee.api.ChatColor;
import org.bukkit.Bukkit;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.plugin.Plugin;

import javax.swing.text.html.parser.Entity;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

/**
 * @BelongsProject: SrWildEntity
 * @BelongsPackage: com.yuyu.srwildentity.config.condition
 * @FileName: BiomeEntity
 * @Author: 峰。
 * @Date: 2024/4/3-15:06
 * @Version: 1.0
 * @Description: 读取实体刷新的配置相关文件
 */
public class BiomeEntityRefreshSettings {
//    private final HashMap<String,List<String>> biomeEntityMap;//此集合储存了群系内需要刷新的怪物
//
//    private final HashMap<String,HashMap<String,EntityCondition>> biomeEntityConditionMap;//此集合储存需要刷新的怪物的相关配置
    private final List<LevelRefresh> levelRefreshesList;
    private final HashMap<String,EntityCondition> generalEntityConditions;
    private int levelMax;


    public BiomeEntityRefreshSettings(Plugin plugin,int levelMax) {

        this.levelRefreshesList = new ArrayList<>();
        this.levelMax = levelMax;
        this.generalEntityConditions = new HashMap<>();


        this.loadRefreshEntity(plugin,levelMax);

    }

    private void loadRefreshEntity(Plugin plugin,int level){
        //保存文件
        plugin.saveResource("biomeEntity.yml",false);

        FileConfiguration config;
        File file = new File(plugin.getDataFolder(), "biomeEntity.yml");
        config = YamlConfiguration.loadConfiguration(file);

        HashMap<String,HashMap<String,Integer>> riskMap = new HashMap<>();
        for (int i = 1;i <= levelMax;i++){
            String risk = config.getString("risk_" + i);
            String[] split = risk.split("\\|");
            int min = Integer.parseInt(split[0]);
            int max = Integer.parseInt(split[1]);
            HashMap<String,Integer> map = new HashMap<>();
            map.put("min",min);
            map.put("max",max);
            riskMap.put("risk_" + i,map);
        }


        //获取所有生物群系
        plugin.saveResource("biome.yml",false);
        FileConfiguration biomeconfig = null;
        File biomeName = new File(plugin.getDataFolder(),"biome.yml");
        biomeconfig = YamlConfiguration.loadConfiguration(biomeName);
        //群系列表
        List<String> biomeList = biomeconfig.getStringList("biome");



        if (config == null || biomeconfig == null){
            plugin.getLogger().info("biomeEntity.yml或者 biome.yml文件读取失败!");
            return;
        }
        //      等级             群系         实体列表
        HashMap<String,HashMap<String,List<String>>> levelBiomeMap = new HashMap<>();

        //按level循环
        for (int i = 1;i <= level;i++){
            HashMap<String,List<String>> biomeMap = new HashMap<>();
            //按群系循环
            for (String biome : biomeList){
                //判断该群系是否存在该等级的配置
                if (config.contains(biome)){
                    //获取到后保存
                    List<String> stringList = config.getStringList(biome + ".ENTITY_LEVEL_" + i);
                    if (stringList.size() > 0) {
                        biomeMap.put(biome, stringList);
                    }
                }
            }
            //按level_i 储存刷新的怪物列表
            levelBiomeMap.put("level_"+i,biomeMap);
        }

        FileConfiguration levelConfig = null;
        File levelFile = new File(plugin.getDataFolder(), "entityCondition.yml");
        levelConfig =  YamlConfiguration.loadConfiguration(levelFile);
        for (int i = 1;i <= level;i++){


            //获取对应等级的怪物刷新列表
            HashMap<String, List<String>> stringListHashMap = levelBiomeMap.get("level_" + i);

            //获取最大,最小危险度  todo(这里放到config.yml里面)
//            int riskMax = levelConfig.getInt("RISK_MAX");
//            int riskMin = levelConfig.getInt("RISK_MIN");
    //                群系         实体列表
            HashMap<String,HashMap<String,EntityCondition>>  listHashMap = new HashMap<>();

            for (String biome : stringListHashMap.keySet()){


//                List<EntityCondition> entityConditions = new ArrayList<>();
                HashMap<String,EntityCondition> entityConditions = new HashMap<>();

                List<String> strings = stringListHashMap.get(biome);
                for (String entityName : strings){

                    String matching_direction = levelConfig.getString(biome + "." + entityName + ".Matching_direction");
                    if (matching_direction != null){
                        String[] split = matching_direction.split("\\|");
                        String formation = split[0];
                        String entity = split[1];

                        EntityCondition entityCondition = makeEntityCondition(entity, formation, levelConfig);
                        entityConditions.put(entityName,entityCondition);
                        plugin.getLogger().info(ChatColor.AQUA +"Level:"+i +"\t"+ entityCondition.toString());
                        continue;
                    }


                    EntityCondition entityCondition = makeEntityCondition(entityName, biome, levelConfig);


                    entityConditions.put(entityName,entityCondition);
                    if (entityCondition == null){
                        plugin.getLogger().info(ChatColor.AQUA +"Level:"+i +"\t"+ entityCondition);
                    }else {
                        plugin.getLogger().info(ChatColor.AQUA + "Level:" + i + "\t" + entityCondition.toString());
                    }
                }

                //此处按群系储存list
                listHashMap.put(biome,entityConditions);
            }

            //此处按等级储存对应等级的LevelRefresh对象
            this.levelRefreshesList.add(new LevelRefresh(riskMap.get("risk_"+i).get("max"),riskMap.get("risk_"+i).get("min"),listHashMap,"等级"+i));
        }



        //循环通用的
        plugin.saveResource("generalCondition.yml",false);
        FileConfiguration generalCondition;
        File generalFile = new File(plugin.getDataFolder(), "generalCondition.yml");
        generalCondition = YamlConfiguration.loadConfiguration(generalFile);
        Set<String> keys = generalCondition.getKeys(false);

        for (String key : keys){
            EntityCondition entityCondition = makeGeneralEntityCondition(key, generalCondition);
            this.generalEntityConditions.put(key,entityCondition);
        }

    }

    public EntityCondition makeEntityCondition(String entityName,String biome,FileConfiguration levelConfig){
        SpawnEntityType spawnEntityType = SpawnEntityType.fromId(levelConfig.getInt(biome + "." + entityName + ".type"));
        EntitySite entitySite = EntitySite.fromId(levelConfig.getInt(biome + "." + entityName + ".site"));
        int light = levelConfig.getInt(biome + "." + entityName + ".light");
        long stime = levelConfig.getLong(biome + "." + entityName + ".startTiming");
        long etime = levelConfig.getLong(biome + "." + entityName + ".endTiming");
        int nums = levelConfig.getInt(biome + "." + entityName + ".nums");
        int yMax = levelConfig.getInt(biome + "." + entityName + ".yMax");
        int yMin = levelConfig.getInt(biome + "." + entityName + ".yMin");
        double weight = levelConfig.getDouble(biome + "." + entityName + ".weight");
        EntityCondition entityCondition = new EntityCondition(entityName, biome,
                spawnEntityType, entitySite, light, stime, etime, nums, yMax, yMin, 50, 0, weight);

        if (entityCondition.getNums()==0){
            Bukkit.getServer().getPluginManager().getPlugin("SrWildEntity").getLogger().info(ChatColor.RED+biome+"."+entityName+".No_Condition");
            return null;
        }
        return entityCondition;
    }

    public EntityCondition makeGeneralEntityCondition(String entityName,FileConfiguration levelConfig){
        SpawnEntityType spawnEntityType = SpawnEntityType.fromId(levelConfig.getInt(entityName + ".type"));
        EntitySite entitySite = EntitySite.fromId(levelConfig.getInt( entityName + ".site"));
        int light = levelConfig.getInt( entityName + ".light");
        long stime = levelConfig.getLong( entityName + ".startTiming");
        long etime = levelConfig.getLong(entityName + ".endTiming");
        int nums = levelConfig.getInt( entityName + ".nums");
        int yMax = levelConfig.getInt(entityName + ".yMax");
        int yMin = levelConfig.getInt( entityName + ".yMin");
        double weight = levelConfig.getDouble( entityName + ".weight");
        EntityCondition entityCondition = new EntityCondition(entityName, null,
                spawnEntityType, entitySite, light, stime, etime, nums, yMax, yMin, 50, 0, weight);

        if (entityCondition.getNums()==0){
            Bukkit.getServer().getPluginManager().getPlugin("SrWildEntity").getLogger().info(ChatColor.RED+"."+entityName+".No_Condition");
            return null;
        }
        return entityCondition;
    }


    public List<LevelRefresh> getLevelRefreshesList() {
        return levelRefreshesList;
    }

    public HashMap<String, EntityCondition> getGeneralEntityConditions() {
        return generalEntityConditions;
    }

    public int getLevelMax() {
        return levelMax;
    }

    public void setLevelMax(int levelMax) {
        this.levelMax = levelMax;
    }
}
